"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint import/no-extraneous-dependencies: 0 */
/* eslint no-console: 0 */
const chalk_1 = __importDefault(require("chalk"));
const printBuildError_1 = __importDefault(require("react-dev-utils/printBuildError"));
const build_wrapper_1 = __importDefault(require("./build-wrapper"));
const client_assets_sizes_1 = require("../util/client-assets-sizes");
const webpack_client_prod_1 = __importDefault(require("../../configs/webpack.client.prod"));
console.log(chalk_1.default.magenta('Building client...'));
(0, build_wrapper_1.default)(webpack_client_prod_1.default)
    .then(({ stats, warnings }) => {
    if (warnings.length) {
        console.log(chalk_1.default.yellow('Client compiled with warnings.\n'));
        console.log(warnings.join('\n\n'));
        console.log(`Search for the ${chalk_1.default.underline(chalk_1.default.yellow('keywords'))} to learn more about each warning.`);
        console.log(`To ignore, add ${chalk_1.default.cyan('// eslint-disable-next-line')} to the line before.`);
    }
    else {
        console.log(chalk_1.default.green('Client compiled successfully.\n'));
    }
    function printOutputSizes(webpackConfig, stats) {
        var _a;
        const sizes = (0, client_assets_sizes_1.calculateAssetsSizes)(stats, (_a = webpackConfig === null || webpackConfig === void 0 ? void 0 : webpackConfig.output) === null || _a === void 0 ? void 0 : _a.path);
        (0, client_assets_sizes_1.printAssetsSizes)(sizes);
    }
    if (Array.isArray(webpack_client_prod_1.default)) {
        webpack_client_prod_1.default.forEach((conf, index) => printOutputSizes(conf, stats.stats[index]));
    }
    else {
        printOutputSizes(webpack_client_prod_1.default, stats);
    }
})
    .catch((err) => {
    console.log(chalk_1.default.red('Failed to compile client.\n'));
    (0, printBuildError_1.default)(err);
    process.exit(1);
});
