"use strict";
/**
 * Формирует описание версии в `CHANGELOG.md` на основе данных введенных в Jenkins при старте пайплайна.
 * `standard-version` запускает этот скрипт как `postchangelog` хук.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const readline_1 = __importDefault(require("readline"));
const app_configs_1 = __importDefault(require("../../configs/app-configs"));
const { readFile, rename, unlink } = fs_1.promises;
/**
 * Сюда пайплайн Jenkins складывает соответствующие части описания версии. Не получается использовать
 * переменные окружения, т.к. `standard-version` запускает хуки через `child_process.exec`, не передавая туда
 * ни аргументы ни переменные окружения.
 */
const { appNodeModulesBin, changelogPath, changelogTmpPath, changelogFeaturesPath, changelogBugfixesPath, changelogBreakingChangesPath } = app_configs_1.default;
// Пример заголовка – `## [50.1.0](http://git.moscow.alfaintra.net/...) (2022-07-27)`.
const changelogHeaderRegExp = /^###? \[\d+\.\d+\.\d+]\([^)]+\) \(\d{4}-\d{2}-\d{2}\)/;
(() => __awaiter(void 0, void 0, void 0, function* () {
    var _a, e_1, _b, _c;
    const rl = readline_1.default.createInterface({
        input: (0, fs_1.createReadStream)(changelogPath),
    });
    const ws = (0, fs_1.createWriteStream)(changelogTmpPath);
    let modifyingStatus = 'PENDING';
    const addVersionDescription = () => __awaiter(void 0, void 0, void 0, function* () {
        const [features, bugFixes, breakingChanges] = yield Promise.all([
            readFile(changelogFeaturesPath, 'utf-8'),
            readFile(changelogBugfixesPath, 'utf-8'),
            readFile(changelogBreakingChangesPath, 'utf-8'),
        ]).then((files) => files.map((content) => content.trim()));
        ws.write(getVersionDescription({ features, bugFixes, breakingChanges }));
    });
    try {
        for (var _d = true, rl_1 = __asyncValues(rl), rl_1_1; rl_1_1 = yield rl_1.next(), _a = rl_1_1.done, !_a;) {
            _c = rl_1_1.value;
            _d = false;
            try {
                const line = _c;
                switch (modifyingStatus) {
                    // Копируем всё как есть до первого заголовка.
                    case 'PENDING':
                        if (changelogHeaderRegExp.test(line)) {
                            modifyingStatus = 'MODIFYING';
                        }
                        ws.write(`${line}\n`);
                        break;
                    // Игнорируем описание версии, сгенерированное пакетом `standard-version`, пишем своё
                    // до второго заголовка. См. описание задачи http://jira.moscow.alfaintra.net/browse/NEWCLICKUI-1048
                    case 'MODIFYING':
                        if (changelogHeaderRegExp.test(line)) {
                            modifyingStatus = 'MODIFIED';
                            yield addVersionDescription();
                            ws.write(`${line}\n`);
                        }
                        break;
                    // Дальше просто копируем всё как есть.
                    case 'MODIFIED':
                        ws.write(`${line}\n`);
                        break;
                }
            }
            finally {
                _d = true;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (!_d && !_a && (_b = rl_1.return)) yield _b.call(rl_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    if (modifyingStatus !== 'MODIFIED') {
        yield addVersionDescription();
    }
    ws.close();
    yield unlink(changelogPath);
    yield rename(changelogTmpPath, changelogPath);
    (0, child_process_1.execSync)(`${appNodeModulesBin}/prettier -w "${changelogPath}"`);
    process.exit(0);
}))();
function getVersionDescription({ features, bugFixes, breakingChanges }) {
    let versionDescription = '';
    if (features) {
        versionDescription += `### Features\n${features}\n`;
    }
    if (bugFixes) {
        versionDescription += `### Bug Fixes\n${bugFixes}\n`;
    }
    if (breakingChanges) {
        versionDescription += `### ⚠ BREAKING CHANGES\n${breakingChanges}\n`;
    }
    return versionDescription;
}
