"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const app_configs_1 = __importDefault(require("../../configs/app-configs"));
const dockerfile_compiled_template_1 = __importDefault(require("../../templates/dockerfile-compiled.template"));
const nginx_conf_template_1 = __importDefault(require("../../templates/nginx.conf.template"));
const start_template_1 = __importDefault(require("../../templates/start.template"));
const exec_1 = __importDefault(require("../util/exec"));
const docker_build_1 = require("../util/docker-build");
(() => __awaiter(void 0, void 0, void 0, function* () {
    const { imageFullName, pathToTempDir, tempDirName } = (0, docker_build_1.getBuildParamsFromArgs)();
    try {
        console.log(`Build docker image ${imageFullName}`);
        console.time('Total time');
        yield (0, docker_build_1.prepareFilesForDocker)({
            pathToTempDir,
            dockerfileTemplate: dockerfile_compiled_template_1.default,
            nginxConfTemplate: nginx_conf_template_1.default,
            startScriptTemplate: start_template_1.default,
            allowLocalDockerfile: false,
            allowLocalStartScript: false,
        });
        yield (0, exec_1.default)('echo "node_modules" >> .dockerignore');
        yield (0, exec_1.default)((0, docker_build_1.getDockerBuildCommand)({ tempDirName, imageFullName }));
        yield fs_extra_1.default.remove(pathToTempDir);
        // guard against pushing the image during tests
        if (!app_configs_1.default.debug) {
            yield (0, exec_1.default)(`docker push ${imageFullName}`);
        }
        console.timeEnd('Total time');
    }
    catch (err) {
        yield fs_extra_1.default.remove(pathToTempDir);
        console.error('Error during docker-build.');
        if (app_configs_1.default.debug) {
            console.error(err);
        }
        process.exit(1);
    }
}))();
