"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDockerBuildCommand = exports.dockerVersionSatisfies = exports.prepareFilesForDocker = exports.getBuildParamsFromArgs = void 0;
const app_configs_1 = __importDefault(require("../../configs/app-configs"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const shelljs_1 = __importDefault(require("shelljs"));
const satisfies_1 = __importDefault(require("semver/functions/satisfies"));
function getBuildParamsFromArgs() {
    let imageVersion = app_configs_1.default.version;
    let imageName = app_configs_1.default.name;
    let dockerRegistry = app_configs_1.default.dockerRegistry;
    const commandLineArguments = process.argv.slice(3);
    commandLineArguments.forEach(arg => {
        let [argName, argValue] = arg.split('=');
        argName = argName.toLowerCase().trim();
        argValue = argValue ? argValue.trim() : '';
        switch (argName) {
            case 'version':
                imageVersion = argValue;
                break;
            case 'name':
                imageName = argValue;
                break;
            case 'registry':
                dockerRegistry = argValue;
                break;
            default:
                console.warn(`Unknown argument ${argName}`);
        }
    });
    const tempDirName = '.docker-build';
    const pathToTempDir = path_1.default.join(app_configs_1.default.cwd, tempDirName);
    const imageFullName = `${dockerRegistry ? `${dockerRegistry}/` : ''}${imageName}:${imageVersion}`;
    return {
        pathToTempDir,
        imageFullName,
        tempDirName,
    };
}
exports.getBuildParamsFromArgs = getBuildParamsFromArgs;
function prepareFilesForDocker({ dockerfileTemplate, nginxConfTemplate, startScriptTemplate, pathToTempDir, allowLocalDockerfile, allowLocalStartScript }) {
    return __awaiter(this, void 0, void 0, function* () {
        yield fs_extra_1.default.emptyDir(pathToTempDir);
        const nginxConf = app_configs_1.default.localNginxConf
            ? yield fs_extra_1.default.readFile(app_configs_1.default.localNginxConf, 'utf8')
            : nginxConfTemplate;
        const dockerfile = app_configs_1.default.localDockerfile && allowLocalDockerfile
            ? yield fs_extra_1.default.readFile(app_configs_1.default.localDockerfile, 'utf8')
            : dockerfileTemplate;
        const startScript = app_configs_1.default.localStartScript && allowLocalStartScript
            ? yield fs_extra_1.default.readFile(app_configs_1.default.localStartScript, 'utf8')
            : startScriptTemplate;
        yield Promise.all([
            fs_extra_1.default.writeFile(path_1.default.join(pathToTempDir, 'Dockerfile'), dockerfile, 'utf8'),
            fs_extra_1.default.writeFile(path_1.default.join(pathToTempDir, 'nginx.conf'), nginxConf, 'utf8'),
            fs_extra_1.default.writeFile(path_1.default.join(pathToTempDir, 'start.sh'), startScript, { encoding: 'utf8', mode: 0o555 }),
        ]);
    });
}
exports.prepareFilesForDocker = prepareFilesForDocker;
function dockerVersionSatisfies(request) {
    const dockerVersion = shelljs_1.default.exec('docker version --format \'{{.Server.Version}}\'', { silent: true });
    return (0, satisfies_1.default)(dockerVersion.toString(), request);
}
exports.dockerVersionSatisfies = dockerVersionSatisfies;
function getDockerBuildCommand({ tempDirName, imageFullName }) {
    // если пытаться собрать проект на маках с m1, докер будет пытаться вытянуть базовый образ под свою платформу и
    // упадет с ошибкой. Чтобы этого избежать - достаточно использовать флаг --platform. Но он поддерживается только начиная
    // с docker 17.12, а на многих серверах, используемых для сборки до сих пор живет докер 1.13.1
    // Соответственно они будут падать при наличии этого флага.
    const canUsePlatformFlag = dockerVersionSatisfies('>=17.12');
    return `docker build ${canUsePlatformFlag ? '--platform linux/x86_64' : ''} -f "./${tempDirName}/Dockerfile" \\
 --build-arg START_SH_LOCATION="./${tempDirName}/start.sh" \\
 --build-arg NGINX_CONF_LOCATION="./${tempDirName}/nginx.conf" -t ${imageFullName} .`;
}
exports.getDockerBuildCommand = getDockerBuildCommand;
