"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCompilers = void 0;
const check_required_files_1 = __importDefault(require("./check-required-files"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const app_configs_1 = __importDefault(require("../../configs/app-configs"));
const child_process_1 = require("child_process");
function runCompilers(pathToCompilers) {
    if (!(0, check_required_files_1.default)()) {
        process.exit(1);
    }
    if (fs_extra_1.default.pathExistsSync(app_configs_1.default.serverOutputPath)) {
        fs_extra_1.default.removeSync(app_configs_1.default.serverOutputPath);
    }
    const compilers = pathToCompilers.map((pathToCompiler) => {
        if (Array.isArray(pathToCompiler)) {
            const compiler = (0, child_process_1.spawn)("node", pathToCompiler, {
                stdio: "inherit",
                cwd: app_configs_1.default.cwd
            });
            compiler.on("error", onProcessExit);
            compiler.on("close", onProcessExit);
            return compiler;
        }
        const compiler = (0, child_process_1.spawn)("node", [pathToCompiler], {
            stdio: "inherit",
        });
        compiler.on("error", onProcessExit);
        compiler.on("close", onProcessExit);
        return compiler;
    });
    function onProcessExit(code) {
        if (code !== 0) {
            compilers.forEach((compiler) => compiler.kill());
            process.exit(code);
        }
    }
}
exports.runCompilers = runCompilers;
