"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstallProductionCommand = exports.getPruningCommand = exports.getYarnVersion = void 0;
const app_configs_1 = __importDefault(require("../../configs/app-configs"));
const shelljs_1 = __importDefault(require("shelljs"));
function getYarnVersion() {
    if (app_configs_1.default.useYarn && shelljs_1.default.which('yarn')) {
        const yarnVersion = shelljs_1.default.exec('yarn -v', { silent: true });
        const yarnMajorVersion = Number(yarnVersion.split('.')[0]);
        return yarnMajorVersion > 1 ? '2+' : '1';
    }
    return 'unavailable';
}
exports.getYarnVersion = getYarnVersion;
function getPruningCommand() {
    const yarnVersion = getYarnVersion();
    switch (yarnVersion) {
        case '1': {
            return 'yarn install --production --ignore-optional --frozen-lockfile --ignore-scripts --prefer-offline';
        }
        case '2+': {
            return 'yarn workspaces focus --production --all';
        }
        case 'unavailable': {
            return 'npm prune --production';
        }
        default: {
            return '';
        }
    }
}
exports.getPruningCommand = getPruningCommand;
function getInstallProductionCommand() {
    const yarnVersion = getYarnVersion();
    switch (yarnVersion) {
        case '1': {
            return 'yarn install --production --ignore-optional --frozen-lockfile --ignore-scripts --prefer-offline';
        }
        case '2+': {
            return 'yarn workspaces focus --production --all';
        }
        case 'unavailable': {
            return 'npm install --production';
        }
        default: {
            return '';
        }
    }
}
exports.getInstallProductionCommand = getInstallProductionCommand;
