"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateDependentContext = exports.calculateDependentConfig = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const get_polyfills_1 = require("../util/get-polyfills");
const resolve_1 = require("../util/resolve");
/**
 * Обновление ключей конфига, зависящих от других. Это нужно делать в самый последний момент
 */
function calculateDependentConfig(config) {
    return Object.assign(Object.assign({}, config), { clientPolyfillsEntry: (0, get_polyfills_1.getPolyfills)(config) });
}
exports.calculateDependentConfig = calculateDependentConfig;
/**
 * Обновление контекста в зависимости от ключей конфига
 */
function calculateDependentContext(config, context) {
    let babelRuntimeVersion;
    try {
        // В проекте может стоять babel/runtime отличный от того, что установлен как зависимость arui-scripts.
        // При этом webpack будет резолвить babel/runtime относительно проекта, а не относительно arui-scripts.
        // Поэтому мы читаем версию того babel/runtime, который установлен в проекте.
        // Так же мы не можем использовать require для получения контента из babel/runtime/package.json,
        // потому что webpack.cache будет ругаться на то, что мы пытаемся импортировать файлы НЕ из своего node_modules.
        const pathToProjectBabelRuntime = (0, resolve_1.resolveNodeModuleRelativeTo)(context.cwd, '@babel/runtime/package.json');
        babelRuntimeVersion = JSON.parse(fs_1.default.readFileSync(pathToProjectBabelRuntime, 'utf8')).version;
    }
    catch (e) {
        babelRuntimeVersion = require('@babel/runtime/package.json').version;
    }
    return Object.assign(Object.assign({}, context), { publicPath: `${config.assetsPath}/`, serverOutputPath: path_1.default.resolve(context.cwd, config.buildPath), clientOutputPath: path_1.default.resolve(context.cwd, config.buildPath, config.assetsPath), statsOutputPath: path_1.default.resolve(context.cwd, config.buildPath, config.statsOutputFilename), watchIgnorePath: ['node_modules', config.buildPath], babelRuntimeVersion });
}
exports.calculateDependentContext = calculateDependentContext;
