"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultAppContext = exports.getDefaultAppConfig = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const resolve_1 = require("../util/resolve");
const CWD = process.cwd();
const absoluteSrcPath = path_1.default.resolve(CWD, 'src');
function getDefaultAppConfig() {
    var _a;
    const appPackage = getPackageJson();
    return {
        /// general settings
        clientServerPort: 8080,
        serverPort: 3000,
        debug: false,
        devSourceMaps: 'eval',
        devServerCors: false,
        useServerHMR: false,
        presets: ((_a = appPackage.aruiScripts) === null || _a === void 0 ? void 0 : _a.presets) || null,
        proxy: appPackage.proxy || null,
        // paths
        buildPath: '.build',
        assetsPath: 'assets',
        additionalBuildPath: ['config'],
        statsOutputFilename: 'stats.json',
        serverEntry: path_1.default.resolve(absoluteSrcPath, 'server/index'),
        serverOutput: 'server.js',
        clientPolyfillsEntry: null,
        clientEntry: path_1.default.resolve(absoluteSrcPath, 'index'),
        // docker compilation configs
        dockerRegistry: '',
        baseDockerImage: 'alfabankui/arui-scripts:latest',
        nginxRootPath: '/src',
        runFromNonRootUser: true,
        removeDevDependenciesDuringDockerBuild: true,
        // archive compilation configs
        archiveName: 'build.tar',
        // build tuning
        keepPropTypes: false,
        useTscLoader: false,
        webpack4Compatibility: false,
        installServerSourceMaps: false,
        disableDevWebpackTypecheck: false,
        // image processing
        dataUrlMaxSize: 1536,
        imageMinimizer: {
            svg: {
                enabled: true,
            },
            gif: {
                enabled: false,
                optimizationLevel: 1,
            },
            jpg: {
                enabled: false,
                quality: 75,
            },
            png: {
                enabled: false,
                optimizationLevel: 3,
                bitDepthReduction: false,
                colorTypeReduction: false,
                paletteReduction: false,
            },
        },
        // CSS
        componentsTheme: null,
        keepCssVars: false,
        // Modules
        compatModules: null,
        modules: null,
    };
}
exports.getDefaultAppConfig = getDefaultAppConfig;
function getPackageJson() {
    const appPackage = JSON.parse(fs_1.default.readFileSync(path_1.default.join(CWD, 'package.json'), 'utf8'));
    if (appPackage['arui-scripts']) {
        throw Error('arui-scripts in package.json is not supported. Use aruiScripts instead.');
    }
    return appPackage;
}
function getDefaultAppContext() {
    const appPackage = getPackageJson();
    const absoluteNodeModulesPath = path_1.default.resolve(CWD, 'node_modules');
    const absoluteNodeModulesBinPath = path_1.default.resolve(absoluteNodeModulesPath, '.bin');
    const projectTsConfigPath = path_1.default.join(CWD, 'tsconfig.json');
    const yarnLockFilePath = path_1.default.join(CWD, 'yarn.lock');
    const overridesPath = (0, resolve_1.tryResolve)(path_1.default.join(CWD, 'arui-scripts.overrides'));
    const nginxConfFilePath = path_1.default.join(CWD, 'nginx.conf');
    const dockerfileFilePath = path_1.default.join(CWD, 'Dockerfile');
    const startScriptFilePath = path_1.default.join(CWD, 'start.sh');
    return {
        appPackage,
        name: appPackage.name,
        normalizedName: appPackage.name.replace(/-/g, '_'),
        version: appPackage.version,
        // general paths
        cwd: CWD,
        appSrc: absoluteSrcPath,
        appNodeModules: absoluteNodeModulesPath,
        appNodeModulesBin: absoluteNodeModulesBinPath,
        overridesPath: overridesPath ? [overridesPath] : [],
        // tools availability
        babelRuntimeVersion: '7.0.0-beta.0',
        useYarn: fs_1.default.existsSync(yarnLockFilePath),
        // compilation configs locations
        tsconfig: fs_1.default.existsSync(projectTsConfigPath) ? projectTsConfigPath : null,
        localNginxConf: fs_1.default.existsSync(nginxConfFilePath) ? nginxConfFilePath : null,
        localDockerfile: fs_1.default.existsSync(dockerfileFilePath) ? dockerfileFilePath : null,
        localStartScript: fs_1.default.existsSync(startScriptFilePath) ? startScriptFilePath : null,
        // Эти пути зависят от других настроек, которые могут быть переопределены пользователем
        publicPath: '',
        serverOutputPath: '',
        clientOutputPath: '',
        statsOutputPath: '',
        watchIgnorePath: [],
        // changelog paths
        changelogPath: path_1.default.resolve(CWD, './CHANGELOG.md'),
        changelogTmpPath: path_1.default.resolve(CWD, './CHANGELOG_TMP.md'),
        changelogFeaturesPath: path_1.default.resolve(CWD, './changelog_features.tmp'),
        changelogBugfixesPath: path_1.default.resolve(CWD, './changelog_bugfixes.tmp'),
        changelogBreakingChangesPath: path_1.default.resolve(CWD, './changelog_breaking_changes.tmp'),
    };
}
exports.getDefaultAppContext = getDefaultAppContext;
