"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateWithPresets = void 0;
const resolve_1 = require("../util/resolve");
const validate_settings_keys_1 = require("./validate-settings-keys");
const lodash_merge_1 = __importDefault(require("lodash.merge"));
function updateWithPresets(config, context) {
    if (!config.presets) {
        return config;
    }
    const presetsConfigPath = (0, resolve_1.tryResolve)(`${config.presets}/arui-scripts.config`, { paths: [context.cwd] });
    const presetsOverridesPath = (0, resolve_1.tryResolve)(`${config.presets}/arui-scripts.overrides`, { paths: [context.cwd] });
    if (presetsConfigPath) {
        let presetsSettings = require(presetsConfigPath);
        if (presetsSettings.__esModule) {
            // ts-node импортирует esModules, из них надо вытягивать default именно так
            presetsSettings = presetsSettings.default;
        }
        (0, validate_settings_keys_1.validateSettingsKeys)(config, presetsSettings, presetsConfigPath);
        config = (0, lodash_merge_1.default)(config, presetsSettings);
    }
    if (presetsOverridesPath) {
        context.overridesPath.unshift(presetsOverridesPath);
    }
    return config;
}
exports.updateWithPresets = updateWithPresets;
