"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchWebpackConfigForCompat = exports.getExposeLoadersFormCompatModules = exports.getCssPrefixForModule = exports.patchMainWebpackConfigForModules = exports.MODULES_ENTRY_NAME = exports.haveExposedDefaultModules = void 0;
const webpack_1 = __importDefault(require("webpack"));
const app_configs_1 = __importDefault(require("./app-configs"));
const find_loader_1 = require("./util/find-loader");
const postcss_prefix_selector_1 = require("../plugins/postcss-prefix-selector");
function haveExposedDefaultModules() {
    var _a;
    return (_a = app_configs_1.default.modules) === null || _a === void 0 ? void 0 : _a.exposes;
}
exports.haveExposedDefaultModules = haveExposedDefaultModules;
exports.MODULES_ENTRY_NAME = 'remoteEntry.js';
function patchMainWebpackConfigForModules(webpackConf) {
    // Добавляем expose loader для библиотек, которые мы хотим вынести в глобальную область видимости
    webpackConf.module.rules.unshift(...getExposeLoadersFormCompatModules());
    if (!app_configs_1.default.modules || !webpackConf.output || !webpackConf.plugins) {
        return webpackConf;
    }
    webpackConf.output.publicPath = haveExposedDefaultModules()
        ? 'auto' // Для того чтобы модули могли подключаться из разных мест, нам необходимо использовать auto. Для корректной работы в IE надо подключaть https://github.com/amiller-gh/currentScript-polyfill
        : app_configs_1.default.publicPath;
    webpackConf.plugins.push(new webpack_1.default.container.ModuleFederationPlugin({
        name: app_configs_1.default.modules.name || app_configs_1.default.normalizedName,
        filename: app_configs_1.default.modules.exposes ? exports.MODULES_ENTRY_NAME : undefined,
        shared: app_configs_1.default.modules.shared,
        exposes: app_configs_1.default.modules.exposes,
    }));
    return webpackConf;
}
exports.patchMainWebpackConfigForModules = patchMainWebpackConfigForModules;
function getCssPrefixForModule(module) {
    if (module.cssPrefix) {
        return module.cssPrefix;
    }
    if (module.cssPrefix === false) {
        return undefined;
    }
    return `.module-${module.name}`;
}
exports.getCssPrefixForModule = getCssPrefixForModule;
function getExposeLoadersFormCompatModules() {
    var _a;
    const shared = (_a = app_configs_1.default.compatModules) === null || _a === void 0 ? void 0 : _a.shared;
    if (!shared) {
        return [];
    }
    return Object.keys(shared).map((libraryName) => {
        const globalVarName = shared[libraryName];
        return {
            test: require.resolve(libraryName),
            use: [
                {
                    loader: require.resolve('expose-loader'),
                    options: {
                        exposes: [globalVarName],
                    },
                },
            ],
        };
    });
}
exports.getExposeLoadersFormCompatModules = getExposeLoadersFormCompatModules;
function addPrefixCssRule(rule, prefix) {
    if (!rule || !rule.use || !Array.isArray(rule.use)) {
        return;
    }
    const postCssLoader = rule.use.find((loaderConfig) => {
        var _a;
        if (typeof loaderConfig === 'string' || typeof loaderConfig === 'function' || !loaderConfig) {
            return false;
        }
        return ((_a = loaderConfig.loader) === null || _a === void 0 ? void 0 : _a.indexOf('postcss-loader')) !== -1;
    });
    if (!postCssLoader || typeof postCssLoader !== 'object' || !postCssLoader.options || typeof postCssLoader.options !== 'object') {
        return;
    }
    postCssLoader.options.postcssOptions.plugins = [
        ...postCssLoader.options.postcssOptions.plugins,
        (0, postcss_prefix_selector_1.postCssPrefix)({ prefix: `${prefix} ` }),
    ];
}
function patchWebpackConfigForCompat(module, webpackConf) {
    webpackConf.externals = Object.assign(Object.assign({}, (webpackConf.externals || {})), (module.externals || {}));
    // Название переменной вебпака, которую он будет использовать для загрузки чанков. Важно чтобы для разных модулей они отличались,
    // иначе несколько модулей из одного приложения будут конфликтовать между собой
    webpackConf.output.uniqueName = module.name;
    const cssPrefix = getCssPrefixForModule(module);
    if (cssPrefix) {
        // Добавляем префикс для css-классов, чтобы изолировать стили модуля от стилей основного приложения
        const cssRule = (0, find_loader_1.findLoader)(webpackConf, '/\\.css$/');
        const cssModulesRule = (0, find_loader_1.findLoader)(webpackConf, '/\\.module\\.css$/');
        addPrefixCssRule(cssRule, cssPrefix);
        addPrefixCssRule(cssModulesRule, `:global(${cssPrefix})`);
    }
    return webpackConf;
}
exports.patchWebpackConfigForCompat = patchWebpackConfigForCompat;
