"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.postcssPluginsOptions = exports.postcssPlugins = exports.createPostcssConfig = void 0;
const app_configs_1 = __importDefault(require("./app-configs"));
const path_1 = __importDefault(require("path"));
/**
 * Функция для создания конфигурационного файла postcss
 * @param {String[]} plugins список плагинов
 * @param {Object} options коллекция конфигураций плагинов, где ключ - название плагина, а значение - аргумент для инициализации
 * @returns {*}
 */
function createPostcssConfig(plugins, options) {
    return plugins.map(pluginName => {
        if (pluginName in options) {
            return [pluginName, options[pluginName]];
        }
        return pluginName;
    });
}
exports.createPostcssConfig = createPostcssConfig;
exports.postcssPlugins = [
    'postcss-omit-import-tilde',
    'postcss-import',
    'postcss-url',
    'postcss-mixins',
    'postcss-for',
    'postcss-each',
    '@csstools/postcss-global-data',
    'postcss-custom-media',
    'postcss-color-mod-function',
    !app_configs_1.default.keepCssVars && 'postcss-custom-properties',
    !app_configs_1.default.keepCssVars && 'postcss-remove-root',
    'postcss-strip-units',
    'postcss-calc',
    'postcss-color-function',
    'postcss-nested',
    'autoprefixer',
    'postcss-inherit',
    'postcss-discard-comments',
].filter(Boolean);
exports.postcssPluginsOptions = {
    'postcss-import': {
        path: ['./src'],
    },
    '@csstools/postcss-global-data': {
        files: [
            path_1.default.join(__dirname, 'mq.css'),
            app_configs_1.default.componentsTheme
        ].filter(Boolean),
    },
    'postcss-url': {
        url: 'rebase',
    },
    'postcss-color-mod-function': {
        unresolved: 'warn',
    },
    'postcss-custom-properties': {
        preserve: false,
    },
};
