import type { Configuration as WebpackConfiguration, WebpackOptionsNormalized } from 'webpack';
import type { Configuration as WebpackDevServerConfiguration } from 'webpack-dev-server';
import { AppContextWithConfigs } from '../app-configs/types';
import { createSingleClientWebpackConfig } from "../webpack.client";
import { findLoader } from './find-loader';
type Overrides = {
    webpack: WebpackConfiguration | WebpackConfiguration[];
    webpackClient: WebpackConfiguration | WebpackConfiguration[];
    webpackDev: WebpackConfiguration | WebpackConfiguration[];
    webpackClientDev: WebpackConfiguration | WebpackConfiguration[];
    webpackServer: WebpackConfiguration;
    webpackServerDev: WebpackConfiguration;
    webpackProd: WebpackConfiguration;
    webpackClientProd: WebpackConfiguration | WebpackConfiguration[];
    webpackServerProd: WebpackConfiguration;
    devServer: WebpackDevServerConfiguration;
    stats: WebpackOptionsNormalized['stats'];
    babel: any;
    babelClient: any;
    babelServer: any;
    babelDependencies: any;
    postcss: any[];
    browsers: string[];
    supportingBrowsers: string[];
    Dockerfile: string;
    'DockerfileCompiled': string;
    nginx: string;
    'start.sh': string;
    serverExternalsExemptions: Array<string | RegExp>;
};
type OmitFirstArg<F> = F extends (x: any, ...args: infer P) => infer R ? (...args: P) => R : never;
type BoundCreateSingleClientWebpackConfig = OmitFirstArg<typeof createSingleClientWebpackConfig>;
type ClientWebpackAdditionalArgs = {
    createSingleClientWebpackConfig: BoundCreateSingleClientWebpackConfig;
    findLoader: typeof findLoader;
};
/**
 * Дополнительные аргументы, которые будут переданы в функцию оверрайда
 */
type OverridesAdditionalArgs = {
    webpack: ClientWebpackAdditionalArgs;
    webpackClient: ClientWebpackAdditionalArgs;
    webpackDev: ClientWebpackAdditionalArgs;
    webpackClientDev: ClientWebpackAdditionalArgs;
};
type OverrideFunction<K extends keyof Overrides, AdditionalArgs = K extends keyof OverridesAdditionalArgs ? OverridesAdditionalArgs[K] : undefined> = (config: Overrides[K], appConfig: AppContextWithConfigs, additionalArgs: AdditionalArgs) => Overrides[K];
export type OverrideFile = {
    [K in keyof Overrides]?: OverrideFunction<K>;
};
/**
 *
 * @param {String|String[]} overridesKey Ключи оверрайда
 * @param {Object} config Конфиг, к которому нужно применить оверрайды
 * @param args Дополнительные аргументы, которые будут переданы в функцию оверрайда
 * @returns {*}
 */
declare function applyOverrides<T extends Overrides[Key], Key extends keyof Overrides, Args = Key extends keyof OverridesAdditionalArgs ? OverridesAdditionalArgs[Key] : undefined>(overridesKey: Key | Key[], config: T, args?: any): T;
export default applyOverrides;
