"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const app_configs_1 = __importDefault(require("../app-configs"));
let overrides = [];
overrides = app_configs_1.default.overridesPath.map(path => {
    try {
        const requireResult = require(path);
        if (requireResult.__esModule) {
            // ts-node импортирует esModules, из них надо вытягивать default именно так
            return requireResult.default;
        }
        return requireResult;
    }
    catch (e) {
        console.error(`Unable to process override file "${path}"`);
        console.log(e);
        return {};
    }
});
/**
 *
 * @param {String|String[]} overridesKey Ключи оверрайда
 * @param {Object} config Конфиг, к которому нужно применить оверрайды
 * @param args Дополнительные аргументы, которые будут переданы в функцию оверрайда
 * @returns {*}
 */
function applyOverrides(overridesKey, config, args) {
    if (typeof overridesKey === 'string') {
        overridesKey = [overridesKey];
    }
    overridesKey.forEach(key => {
        overrides.forEach((override) => {
            if (override.hasOwnProperty(key)) {
                const overrideFn = override[key];
                if (typeof overrideFn !== 'function') {
                    throw new TypeError(`Override ${key} must be a function`);
                }
                config = overrideFn(config, app_configs_1.default, args);
            }
        });
    });
    return config;
}
exports.default = applyOverrides;
