"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @param {string|string[]|object} entryPoint Строка, массив строк или объект с энтрипоинтами
 * @param {Function} getSingleEntry Функция, возвращающая конфигурацию для одного entryPoint
 * @param args Дополнительные аргументы, которые будут переданы в getSingleEntry функцию
 * @returns {*}
 */
function getEntry(entryPoint, getSingleEntry, ...args) {
    if (typeof entryPoint === 'string') {
        return getSingleEntry([entryPoint], ...args);
    }
    if (Array.isArray(entryPoint)) {
        return getSingleEntry(entryPoint, ...args);
    }
    // client entry also can be an object, so we must add hot loader to each entry point
    return Object.keys(entryPoint).reduce((result, entryPointName) => {
        const entry = getEntry(entryPoint[entryPointName], getSingleEntry, ...args);
        return Object.assign(Object.assign({}, result), { [entryPointName]: entry });
    }, {});
}
exports.default = getEntry;
