"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createClientWebpackConfig = exports.createSingleClientWebpackConfig = void 0;
const path_1 = __importDefault(require("path"));
const webpack_1 = __importDefault(require("webpack"));
const webpack_manifest_plugin_1 = require("webpack-manifest-plugin");
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const terser_webpack_plugin_1 = __importDefault(require("terser-webpack-plugin"));
const fork_ts_checker_webpack_plugin_1 = __importDefault(require("fork-ts-checker-webpack-plugin"));
const compression_webpack_plugin_1 = __importDefault(require("compression-webpack-plugin"));
const assets_webpack_plugin_1 = __importDefault(require("assets-webpack-plugin"));
const tsconfig_paths_webpack_plugin_1 = __importDefault(require("tsconfig-paths-webpack-plugin"));
const getCSSModuleLocalIdent_1 = __importDefault(require("react-dev-utils/getCSSModuleLocalIdent"));
const get_entry_1 = __importDefault(require("./util/get-entry"));
const app_configs_1 = __importDefault(require("./app-configs"));
const babel_client_1 = __importDefault(require("./babel-client"));
const postcss_1 = __importDefault(require("./postcss"));
const check_node_version_1 = __importDefault(require("./util/check-node-version"));
const babel_dependencies_1 = require("./babel-dependencies");
const react_refresh_typescript_1 = __importDefault(require("react-refresh-typescript"));
const react_refresh_webpack_plugin_1 = __importDefault(require("@pmmmwh/react-refresh-webpack-plugin"));
const case_sensitive_paths_webpack_plugin_1 = __importDefault(require("case-sensitive-paths-webpack-plugin"));
const webpack_deduplication_plugin_1 = require("webpack-deduplication-plugin");
const minimizers_1 = require("./config-extras/minimizers");
const mini_svg_data_uri_1 = __importDefault(require("mini-svg-data-uri"));
const process_assets_plugin_output_1 = require("./process-assets-plugin-output");
const modules_1 = require("./modules");
const PnpWebpackPlugin = require('pnp-webpack-plugin');
const CssMinimizerPlugin = require('css-minimizer-webpack-plugin');
const noopPath = require.resolve('./util/noop');
function getSingleEntry(entryPoint, mode) {
    return [
        ...(Array.isArray(app_configs_1.default.clientPolyfillsEntry) ? app_configs_1.default.clientPolyfillsEntry : [app_configs_1.default.clientPolyfillsEntry]),
        ...entryPoint
    ].filter(Boolean);
}
// Нам важно шарить assets plugin между разными сборками, чтобы файл не перезаписывался
const assetsPlugin = new assets_webpack_plugin_1.default({
    path: app_configs_1.default.serverOutputPath,
    processOutput: process_assets_plugin_output_1.processAssetsPluginOutput,
});
// Этот плагин нужен для того, чтобы клиентский код так же могу получить информацию о сборке.
// Это позволяет нам использовать данные о сборки на клиенте.
const clientAssetsPlugin = new assets_webpack_plugin_1.default({
    path: app_configs_1.default.clientOutputPath,
    processOutput: process_assets_plugin_output_1.processAssetsPluginOutput,
});
function getMinimizeConfig(mode) {
    if (mode === 'prod') {
        return {
            minimize: true,
            minimizer: [
                ...(0, minimizers_1.getImageMin)(),
                new terser_webpack_plugin_1.default({
                    terserOptions: {
                        parse: {
                            // we want terser to parse ecma 8 code. However, we don't want it
                            // to apply any minfication steps that turns valid ecma 5 code
                            // into invalid ecma 5 code. This is why the 'compress' and 'output'
                            // sections only apply transformations that are ecma 5 safe
                            // https://github.com/facebook/create-react-app/pull/4234
                            ecma: 2017,
                        },
                        compress: {
                            ecma: 5,
                            // Disabled because of an issue with Uglify breaking seemingly valid code:
                            // https://github.com/facebook/create-react-app/issues/2376
                            // Pending further investigation:
                            // https://github.com/mishoo/UglifyJS2/issues/2011
                            comparisons: false,
                            // Disabled because of an issue with Terser breaking valid code:
                            // https://github.com/facebook/create-react-app/issues/5250
                            // Pending futher investigation:
                            // https://github.com/terser-js/terser/issues/120
                            inline: 2,
                        },
                        mangle: {
                            safari10: true,
                        },
                        output: {
                            ecma: 5,
                            comments: false,
                            // Turned on because emoji and regex is not minified properly using default
                            // https://github.com/facebook/create-react-app/issues/2488
                            ascii_only: true,
                        },
                    },
                    // Use multi-process parallel running to improve the build speed
                    // Default number of concurrent runs: os.cpus().length - 1
                    parallel: true,
                }),
                new CssMinimizerPlugin()
            ].filter(Boolean)
        };
    }
    if (app_configs_1.default.keepCssVars) {
        // Каждый импорт css файла с переменными приводит к дублированию всего набора.
        // Чтобы этого избежать — проходимся плагином, который удалит дубли из выходного файла.
        return {
            minimize: true,
            minimizer: [
                new CssMinimizerPlugin({
                    minimizerOptions: {
                        preset: () => ({
                            plugins: [require('postcss-discard-duplicates')],
                        }),
                    },
                }),
            ]
        };
    }
    return {
        minimize: false,
        minimizer: []
    };
}
/**
 * Создает конфигурацию для сборки клиентского кода. Может быть использована для создания дополнительных конфигураций.
 * @param mode Режим сборки (dev или prod)
 * @param entry Точка входа, любой валидный вход для webpack
 * @param configName Имя конфигурации, если не указано, то используется имя по умолчанию
 */
const createSingleClientWebpackConfig = (mode, entry, configName) => (Object.assign(Object.assign({}, (configName ? { name: configName } : {})), { target: 'web', mode: mode === 'dev' ? 'development' : 'production', devtool: mode === 'dev' ? app_configs_1.default.devSourceMaps : 'source-map', entry: (0, get_entry_1.default)(entry, getSingleEntry, mode), 
    // in production mode we need to fail on first error
    bail: mode === 'prod', context: app_configs_1.default.cwd, output: {
        assetModuleFilename: 'static/media/[name].[hash:8][ext]',
        // Add /* filename */ comments to generated require()s in the output.
        pathinfo: mode === 'dev',
        path: app_configs_1.default.clientOutputPath,
        publicPath: app_configs_1.default.publicPath,
        filename: mode === 'dev' ? '[name].js' : '[name].[chunkhash:8].js',
        chunkFilename: mode === 'dev'
            ? `${configName ? `${configName}-` : ''}[name].js`
            : `${configName ? `${configName}-` : ''}[name].[chunkhash:8].chunk.js`,
        // Point sourcemap entries to original disk location (format as URL on Windows)
        devtoolModuleFilenameTemplate: (info) => path_1.default
            .relative(app_configs_1.default.appSrc, info.absoluteResourcePath)
            .replace(/\\/g, '/'),
    }, optimization: Object.assign(Object.assign({}, getMinimizeConfig(mode)), { splitChunks: mode === 'prod'
            ? {
                chunks: "all",
                cacheGroups: {
                    commons: {
                        chunks: "initial",
                        minChunks: 2,
                        maxInitialRequests: 5,
                        minSize: 0 // This is example is too small to create commons chunks
                    },
                    vendor: {
                        test: /node_modules/,
                        chunks: "initial",
                        name: (_, chunks, cacheGroupKey) => {
                            if (!configName) {
                                return 'vendor';
                            }
                            // Нам нужно сделать так, чтобы у разных конфигураций были разные имена чанков
                            const allChunksNames = chunks.map((item) => item.name).join('~');
                            return `${cacheGroupKey}-${allChunksNames}`;
                        },
                        priority: 10,
                        enforce: true
                    }
                }
            }
            : false, nodeEnv: mode === 'prod' ? 'production' : false, 
        // Оптимизации времени билда, см https://webpack.js.org/guides/build-performance/
        removeAvailableModules: mode !== 'dev', removeEmptyChunks: mode !== 'dev' }), resolve: {
        // This allows you to set a fallback for where Webpack should look for modules.
        // We placed these paths second because we want `node_modules` to "win"
        // if there are any conflicts. This matches Node resolution mechanism.
        // https://github.com/facebookincubator/create-react-app/issues/253
        modules: ['node_modules', app_configs_1.default.appNodeModules],
        // These are the reasonable defaults supported by the Node ecosystem.
        // We also include JSX as a common component filename extension to support
        // some tools, although we do not recommend using it, see:
        // https://github.com/facebookincubator/create-react-app/issues/290
        // `web` extension prefixes have been added for better support
        // for React Native Web.
        extensions: ['.web.js', '.mjs', '.cjs', '.js', '.json', '.web.jsx', '.jsx', '.ts', '.tsx'],
        plugins: ([
            (app_configs_1.default.tsconfig && new tsconfig_paths_webpack_plugin_1.default({
                configFile: app_configs_1.default.tsconfig,
                extensions: ['.web.js', '.mjs', '.js', '.json', '.web.jsx', '.jsx', '.ts', '.tsx']
            })),
        ].filter(Boolean)),
    }, resolveLoader: {
        plugins: [
            PnpWebpackPlugin.moduleLoader(module),
        ],
    }, cache: mode === 'dev'
        ? {
            type: 'filesystem',
            buildDependencies: {
                config: [__filename],
            },
        }
        : false, module: {
        // typescript interface will be removed from modules, and we will get an error on correct code
        // see https://github.com/webpack/webpack/issues/7378
        strictExportPresence: !app_configs_1.default.tsconfig,
        rules: [
            {
                // "oneOf" will traverse all following loaders until one will
                // match the requirements. When no loader matches it will fall
                // back to the "file" loader at the end of the loader list.
                oneOf: ([
                    // Process JS with Babel.
                    {
                        test: app_configs_1.default.useTscLoader ? /\.(js|jsx|mjs|cjs)$/ : /\.(js|jsx|mjs|ts|tsx|cjs)$/,
                        include: app_configs_1.default.appSrc,
                        loader: require.resolve('babel-loader'),
                        options: Object.assign(Object.assign({}, babel_client_1.default), { cacheDirectory: mode === 'dev', cacheCompression: false, plugins: [
                                ...babel_client_1.default.plugins,
                                mode === 'dev' ? [require.resolve('react-refresh/babel'), { skipEnvCheck: true }] : undefined
                            ].filter(Boolean) }),
                    },
                    (app_configs_1.default.tsconfig && app_configs_1.default.useTscLoader) && {
                        test: /\.tsx?$/,
                        use: [
                            {
                                loader: require.resolve('babel-loader'),
                                options: Object.assign({
                                    cacheDirectory: mode === 'dev',
                                    cacheCompression: false,
                                    plugins: mode === 'dev' ? [require.resolve('react-refresh/babel'), { skipEnvCheck: true }] : undefined,
                                }, babel_client_1.default)
                            },
                            {
                                loader: require.resolve('ts-loader'),
                                options: {
                                    getCustomTransformers: () => ({
                                        before: mode === 'dev' ? [(0, react_refresh_typescript_1.default)()] : [],
                                    }),
                                    onlyCompileBundledFiles: true,
                                    transpileOnly: true,
                                    happyPackMode: true,
                                    configFile: app_configs_1.default.tsconfig
                                }
                            }
                        ]
                    },
                    // Process any JS outside of the app with Babel.
                    // Unlike the application JS, we only compile the standard ES features.
                    {
                        test: /\.(js|mjs)$/,
                        exclude: /@babel(?:\/|\\{1,2})runtime/,
                        loader: require.resolve('babel-loader'),
                        options: Object.assign(Object.assign({}, babel_dependencies_1.babelDependencies), { babelrc: false, configFile: false, compact: false, cacheDirectory: mode === 'dev', cacheCompression: false }),
                    },
                    // process simple css files with postcss loader and css loader
                    {
                        test: /\.css$/,
                        exclude: /\.module\.css$/,
                        use: [
                            {
                                loader: mini_css_extract_plugin_1.default.loader,
                                options: { publicPath: './' }
                            },
                            {
                                loader: require.resolve('css-loader'),
                                options: {
                                    importLoaders: 1,
                                    sourceMap: mode === 'dev',
                                },
                            },
                            {
                                loader: require.resolve('postcss-loader'),
                                options: {
                                    postcssOptions: {
                                        plugins: postcss_1.default,
                                    }
                                },
                            },
                        ],
                    },
                    // process all *.module.css files with postcss loader and css loader
                    {
                        test: /\.module\.css$/,
                        use: [
                            {
                                loader: mini_css_extract_plugin_1.default.loader,
                                options: { publicPath: './' }
                            },
                            {
                                loader: require.resolve('css-loader'),
                                options: {
                                    importLoaders: 1,
                                    sourceMap: mode === 'dev',
                                    modules: {
                                        getLocalIdent: getCSSModuleLocalIdent_1.default
                                    },
                                },
                            },
                            {
                                loader: require.resolve('postcss-loader'),
                                options: {
                                    postcssOptions: {
                                        plugins: postcss_1.default,
                                    }
                                },
                            },
                        ],
                    },
                    {
                        test: /\.svg/,
                        type: 'asset',
                        generator: {
                            dataUrl: (content) => (0, mini_svg_data_uri_1.default)(content.toString())
                        },
                        parser: {
                            dataUrlCondition: {
                                maxSize: app_configs_1.default.dataUrlMaxSize
                            }
                        }
                    },
                    {
                        exclude: [/\.(js|jsx|mjs|cjs|ts|tsx)$/, /\.(html|ejs)$/, /\.json$/],
                        type: 'asset',
                        parser: {
                            dataUrlCondition: {
                                maxSize: app_configs_1.default.dataUrlMaxSize
                            }
                        }
                    },
                ].filter(Boolean)),
            },
            // ** STOP ** Are you adding a new loader?
            // Make sure to add the new loader(s) before asset modules
        ],
    }, plugins: ([
        assetsPlugin,
        clientAssetsPlugin,
        new webpack_1.default.DefinePlugin(Object.assign({ 
            // Tell Webpack to provide empty mocks for process.env.
            'process.env': '{}' }, (mode === 'dev'
            ? { 'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV) }
            : {}))),
        new mini_css_extract_plugin_1.default({
            ignoreOrder: true,
            filename: mode === 'dev' ? '[name].css' : '[name].[contenthash:8].css',
            chunkFilename: mode === 'dev' ? '[id].css' : '[name].[contenthash:8].chunk.css',
        }),
        (mode === "prod" || !app_configs_1.default.disableDevWebpackTypecheck) && app_configs_1.default.tsconfig !== null && new fork_ts_checker_webpack_plugin_1.default(),
        // moment.js очень большая библиотека, которая включает в себя массу локализаций, которые мы не используем.
        // Поэтому мы их просто игнорируем, чтобы не включать в сборку.
        // https://github.com/jmblog/how-to-optimize-momentjs-with-webpack
        new webpack_1.default.IgnorePlugin({
            resourceRegExp: /^\.\/locale$/,
            contextRegExp: /moment$/,
        }),
        // Этот плагин позволяет избавиться от одинаковых зависимостей, которые могут появляться из-за того, что одна и та же
        // библиотека является зависимостью в разных версиях для разных библиотек.
        // например проект зависит от lodash@1, библиотеки A и B зависят от lodash@2. Тогда lodash@2 будет включен в сборку
        // дважды, что приведет к увеличению размера бандла (и это не лечится дедубликацией в yarn/npm).
        // Сама реализация плагина зависит от yarn.lock, поэтому если мы используем npm, то плагин не будет работать.
        app_configs_1.default.useYarn && new webpack_deduplication_plugin_1.WebpackDeduplicationPlugin({
            cacheDir: path_1.default.join(app_configs_1.default.cwd, 'node_modules/.cache/deduplication-webpack-plugin/'),
            rootPath: app_configs_1.default.cwd,
        }),
        // dev plugins:
        mode === 'dev' && new react_refresh_webpack_plugin_1.default({
            overlay: false,
        }),
        // Watcher doesn't work well if you mistype casing in a path so we use
        // a plugin that prints an error when you attempt to do this.
        // See https://github.com/facebookincubator/create-react-app/issues/240
        mode === 'dev' && new case_sensitive_paths_webpack_plugin_1.default(),
        // production plugins:
        mode === 'prod' && new webpack_manifest_plugin_1.WebpackManifestPlugin(),
        mode === 'prod' && new compression_webpack_plugin_1.default({
            filename: '[file].gz',
            algorithm: 'gzip',
            test: /\.js$|\.css$|\.png$|\.svg$/,
            threshold: 10240,
            minRatio: 0.8
        }),
        mode === 'prod' && (0, check_node_version_1.default)(10) && new compression_webpack_plugin_1.default({
            filename: '[file].br',
            algorithm: 'brotliCompress',
            test: /\.(js|css|html|svg)$/,
            compressionOptions: {
                level: 11,
            },
            threshold: 10240,
            minRatio: 0.8,
        }),
        // Ignore prop-types packages in production mode
        // This should works fine, since proptypes usage should be eliminated in production mode
        mode === 'prod' && !app_configs_1.default.keepPropTypes && new webpack_1.default.NormalModuleReplacementPlugin(/^react-style-proptype$/, noopPath),
        mode === 'prod' && !app_configs_1.default.keepPropTypes && new webpack_1.default.NormalModuleReplacementPlugin(/^thrift-services\/proptypes/, noopPath)
    ].filter(Boolean)), experiments: {
        backCompat: app_configs_1.default.webpack4Compatibility,
    }, 
    // Без этого комиляция трирегилась на изменение в node_modules и приводила к утечке памяти
    watchOptions: {
        ignored: new RegExp(app_configs_1.default.watchIgnorePath.join('|')),
        aggregateTimeout: 100,
    }, 
    // Выключаем performance hints, т.к. размеры бандлов контролируются не в рамках arui-scripts
    performance: {
        hints: false,
    } }));
exports.createSingleClientWebpackConfig = createSingleClientWebpackConfig;
const createClientWebpackConfig = (mode) => {
    var _a;
    const appWebpackConfig = (0, modules_1.patchMainWebpackConfigForModules)((0, exports.createSingleClientWebpackConfig)(mode, app_configs_1.default.clientEntry));
    const exposedCompatModules = (_a = app_configs_1.default.compatModules) === null || _a === void 0 ? void 0 : _a.exposes;
    if (!exposedCompatModules || Object.keys(exposedCompatModules).length === 0) {
        return appWebpackConfig;
    }
    // Добавляем отдельные конфигурации для compat модулей
    const modulesWebpackConfigs = Object.keys(exposedCompatModules).map((moduleName) => {
        const module = Object.assign({ name: moduleName }, exposedCompatModules[moduleName]);
        const config = (0, exports.createSingleClientWebpackConfig)(mode, {
            [module.name]: module.entry,
        }, module.name);
        return (0, modules_1.patchWebpackConfigForCompat)(module, config);
    });
    return [appWebpackConfig, ...modulesWebpackConfigs];
};
exports.createClientWebpackConfig = createClientWebpackConfig;
