"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const app_configs_1 = __importDefault(require("../configs/app-configs"));
const apply_overrides_1 = __importDefault(require("../configs/util/apply-overrides"));
const yarn_1 = require("../commands/util/yarn");
const installProductionCommand = (0, yarn_1.getInstallProductionCommand)();
const yarnVersion = (0, yarn_1.getYarnVersion)();
// В зависимости от используемого мендежера зависимостей для их установки нужно копировать разный набор файлов
const filesRequiredToInstallDependencies = [
    'package.json',
    'yarn.lock',
    yarnVersion === '2+' && '.yarnrc.yml',
    yarnVersion === '2+' && '.yarn',
    yarnVersion === 'unavailable' && 'package-lock.json',
].filter(Boolean);
const dockerfileTemplate = `
FROM ${app_configs_1.default.baseDockerImage}
ARG START_SH_LOCATION
ARG NGINX_CONF_LOCATION

WORKDIR /src

# Полу-статичные файлы, могут легко кешироваться
ADD $START_SH_LOCATION /src/start.sh
ADD $NGINX_CONF_LOCATION /src/nginx.conf

# Зависимости. При некоторой удаче могут кешироваться и соответственно кешировать установку зависимостей
${filesRequiredToInstallDependencies.map((file) => `ADD --chown=nginx:nginx ${file} /src/${file}`).join('\n')}

RUN ${installProductionCommand}  && \\
    ${yarnVersion === 'unavailable' ? 'npm cache clean --force' : 'yarn cache clean'}

ADD --chown=nginx:nginx . /src

# Создаем директории для nginx и выставляем правильные права
RUN mkdir -p /var/lib/nginx && \
    chown -R nginx:nginx /var/lib/nginx && \
    chown -R nginx:nginx /var/log/nginx && \
    chown -R nginx:nginx /etc/nginx/conf.d
RUN touch /var/run/nginx.pid && \
   chown -R nginx:nginx /var/run/nginx.pid
USER nginx
`;
exports.default = (0, apply_overrides_1.default)('DockerfileCompiled', dockerfileTemplate);
