"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const app_configs_1 = __importDefault(require("../configs/app-configs"));
const apply_overrides_1 = __importDefault(require("../configs/util/apply-overrides"));
const startTemplate = `#!/bin/sh

# Подменяем env переменные в nginx конфиге перед стартом
# Сначала заменяем все слова, начинающиеся на $ но без \${} на ~~слово~~
# затем запускаем envsubst, после этого обратно заменяем ~~слово~~ на $слово.
# Это нужно для того, чтоб специальные переменные nginx не подменялись envsubst'ом на
# пустые строки. envsubst будет заменять только \${слово}.
cat ./nginx.conf \\
    | sed 's/\\$\\([a-zA-Z0-9_-]\\{1,\\}\\)/~~\\1~~/g' \\
    | envsubst \\
    | sed 's/~~\\([a-zA-Z0-9_-]\\{1,\\}\\)~~/$\\1/g' \\
    > /etc/nginx/conf.d/default.conf

# Достаем лимит памяти из cgroup, это то, как его докер задает.
if [[ -f /sys/fs/cgroup/cgroup.controllers ]]; then
   max_total_memory=$(cat /sys/fs/cgroup/memory.max)
else
   max_total_memory=$(cat /sys/fs/cgroup/memory/memory.limit_in_bytes)
fi
# Самой nodejs мы не можем отдать совсем всю память, операционная система+nginx все же требуют какого то количества.
# Поэтому мы вычитаем 100мб на нужды ос
node_memory_limit="$(($max_total_memory / 1024 / 1024 - 100))"

# Start the nginx process in background
nginx &

# Start nodejs process
node --max-old-space-size="$node_memory_limit" ./${app_configs_1.default.buildPath}/${app_configs_1.default.serverOutput}
`;
exports.default = (0, apply_overrides_1.default)('start.sh', startTemplate);
