Настройки сборки артефакта
===

## docker

По-умолчанию, базовым образом для сборки артефакта является [alpine-node-nginx](../../alpine-node-nginx).

На `8080` порту будет поднят nginx, который будет раздавать статику и проксировать все остальные запросы к `nodejs`.

Вы также можете переопределить полностью процесс сборки docker-образа используя механизм [overrides](#тонкая-настройка)
или создав в корневой директории проекта `Dockerfile` содержащий необходимый набор инструкций.
Пример [Dockerfile](src/templates/dockerfile.template.ts).

`Dockerfile` в корне проекта имеет приоритет над overrides.

Чтобы переопределить скрипт запуска, воспользуйтесь механизмом [overrides](#тонкая-настройка)
или создайте в корневой директории проекта `start.sh` файл содержащий необходимый набор инструкций.
Пример [start.sh](src/templates/start.template.ts).

`start.sh` в корне проекта имеет приоритет над overrides.


## Концигурация nginx
Несмотря на то, что nginx имеет готовый конфиг с роутингом, иногда возникает необходимость добавлять свои роуты.
Вы можете использовать механизм [overrides](overrides.md).
Так же вы можете создать `nginx.conf` на уровне проекта со своими роутами. Пример конфига [тут](../src/templates/nginx.conf.template.ts).
Файл nginx.conf имеет приоритет над оверрайдами.

### Использование env переменных в nginx.conf

Иногда у вас может возникнуть потребность переопределять какие-то из настроек nginx в зависимости
от среды, на которой запущен контейнер. Это можно сделать задав свой `nginx.conf` и передав ENV переменные
в контейнер. По умолчанию конфигурация nginx прогоняется при старте
через [envsubst](https://www.gnu.org/software/gettext/manual/html_node/envsubst-Invocation.html).

Если вы используете свой базовый docker-образ для работы приложения - убедитесь что в нем доступен `envsubst`.
Для alpine он является частью пакета [`gettext`](https://pkgs.alpinelinux.org/contents?branch=edge&name=gettext&arch=x86&repo=main)

**Важно**. Для того чтобы сохранить нормальную работу специальных переменных nginx, типа `$proxy_add_x_forwarded_for`
перед запуском envsubst они будут заменены на `~~proxy_add_x_forwarded_for~~`, а затем возвращены в исходный вид.
envsubst будет заменять переменные записанные **только** как `${MY_VAR}`.


Вы можете использовать это так:

```nginx.conf
server {
    listen 8080;
    server_name ${SERVICE_NAME};
    ...
}
```

```shell
docker run my-awesome-app --env SERVICE_NAME=my-app
```

После запуска nginx будет иметь server_name `my-app`.
